(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 10.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     43501,       1223]
NotebookOptionsPosition[     41268,       1155]
NotebookOutlinePosition[     41627,       1171]
CellTagsIndexPosition[     41584,       1168]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["\<\
Jordan algebra:

Commutative product from anticommutator of general product
Non-associative in general, but power-associative:
(x^m.y).x^n = x^m.(y.x^n)

Automorphism of a Lie algebra: derivation of the Jordan algebra \
\[OpenCurlyDoubleQuote]der\[CloseCurlyDoubleQuote]: D(x*y) = (D(x))*y + \
x*(D(y))

Direct product with the Jordan algebra, the structure or \
\[OpenCurlyDoubleQuote]str\[CloseCurlyDoubleQuote].


Formally-real Jordan algebras here, where the sum of squares of a\
\[CloseCurlyQuote]s = 0 means that they are all 0. Every one can be \
decomposed into a sum of the simple ones discussed here.


Spin-factor Jordan algebra:
It operates on (scalar,vector) -- R * R^n
(xs,xv) * (ys,yv) = (xs*ys + xv.yv, xs*yv + ys*xv)

h(n,X) -- for n*n Hermitian matrices of division algebra X, with that algebra\
\[CloseCurlyQuote]s multiplication rule used:
{{real, x}, {conjugate(x), real}}

h(1,X) = real multiplication -- trivial

h(2,X) = spin factor

Proof:
M(x) = {{x0+x1, x2}, {x2c, x0-x1}}
c = conjugate
M(z) = (1/2)(M(x).M(y) + M(y).M(x))
gives us
z0+z1 = (x0+x1)*(y0+y1) + (1/2)(x2*y2c + y2*x2c)
z0-z1 = (x0-z1)*(y0-y1) + (1/2)*(x2c*y2 + y2c*x2)
x2 = x0*y2 + y0*x2

x*yc + y*xc = (1/2)( (x+y)*(xc+yc) - (x-y)*(xc-yc) ) = (1/2)( (x+y).(x+y) - \
(x-y).(x-y) ) = 2(x.y)
from norm = inner product of unrolled obect
Likewise for xc*y + yc*x

Thus,
z0 = x0*y0 + x1*y1 + x2.y2
x1 = x0*y1 + y0*x1
x2 = x0*y2 + y0*x2
Scalar part: x0
Vector part: x1 with x2

h(3,X) -- X is R, C, H, O
H(n,X), n>3 -- X is R, C, H

Automorphism groups:

Spin factor for scalar and n-vector: SO(n)
h(1,X) = none
h(2,X) = spin factor with (2^np+1) - vector
h(n,R) -- n(n-1)/2 -- SO(n)
h(n,C) -- n^2-1 -- SU(n)
h(n,H) -- n(2n+1) -- Sp(2n)
h(2,O) -- 36 -- SO(9)
h(3,O) -- 52 -- F4\
\>", "Text",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGCQAGIQ/WByk2bGr1eOii9+2YLoCRzmziC6juuqC4juflAS
CaJ3XnRNBtG2UiFqmUD6ls0MTRDdVGZtDqJ1p1dZgegFrF+mgGjJedHTQfQz
S7v8LJB5pnMKQHTErkNzQLSSnPA8EP2Gc5rWfiDt7LHOEUR/SQzxANFcS5oK
QfSNFZymR4B09YYj1iA6wfxq5w8gfazuQy+IDovqWgCipapmgekC++QlIFo3
wHApiL7UcWQtiP7VFbkOROemrtoMouWcz+0B0d+kWnR/AmmtlhRTEM0Yt8EK
RO9bMtMWRN/be9wRRLv0mLuC6F4piwgQrSYgHAOi/+wSSwDRM0p9E0F0+9JP
0X+A9IYergQQfU9oVSqIltuWVwaiw97Ntfz3+5Wj4ZvlYBoAnArkjw==
  "]],

Cell[CellGroupData[{

Cell["Division-Algebra Setup", "Section",
 CellChangeTimes->{{3.6209756500785847`*^9, 3.620975660084325*^9}, {
  3.620975696292815*^9, 3.620975698109521*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Division", "-", 
   RowBox[{"algebra", " ", "conjugate"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.620975677960123*^9, 3.620975682878397*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDConj", "[", "x_", "]"}], " ", ":=", " ", 
  RowBox[{"Prepend", "[", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Rest", "[", "x", "]"}]}], ",", 
    RowBox[{"First", "[", "x", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.620975707450385*^9, 3.620975715734764*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Division", "-", 
   RowBox[{"algebra", " ", "unit", " ", "multiplication", " ", "table"}]}], 
  " ", "*)"}]], "Input",
 CellChangeTimes->{{3.62097501938091*^9, 3.620975037807354*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDUnitMultStep", "[", "data_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "np", ",", "rls", ",", "newnp", " ", ",", "newrls", ",", "n", ",", "ih1",
       ",", "il1", ",", "ih2", ",", "il2", ",", "i1", ",", "i2", ",", "i12", 
      ",", "s"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"np", ",", "rls"}], "}"}], " ", "=", " ", "data"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"newnp", " ", "=", " ", 
      RowBox[{"np", "+", "1"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"2", "^", "np"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"newrls", " ", "=", " ", 
      RowBox[{"Sort", "[", 
       RowBox[{"Flatten", "[", 
        RowBox[{"Table", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"i1", " ", "=", " ", 
            RowBox[{
             RowBox[{"n", "*", "ih1"}], "+", "il1"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"i2", " ", "=", " ", 
            RowBox[{
             RowBox[{"n", "*", "ih2"}], "+", "il2"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"i12", ",", "s"}], "}"}], " ", "=", " ", 
            RowBox[{
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"ih2", ">", "0"}], ",", 
               RowBox[{"{", 
                RowBox[{"il2", ",", "il1"}], "}"}], " ", ",", " ", 
               RowBox[{"{", 
                RowBox[{"il1", ",", "il2"}], "}"}]}], "]"}], " ", "/.", " ", 
             "rls"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"ih1", "\[NotEqual]", "ih2"}], ",", 
             RowBox[{"i12", "+=", "n"}]}], "]"}], ";", "\[IndentingNewLine]", 
           
           RowBox[{"s", " ", "*=", " ", 
            RowBox[{
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"ih1", ">", "0"}], ",", "  ", 
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{"il2", "\[Equal]", "0"}], ",", " ", "1", ",", " ", 
                 RowBox[{"-", "1"}]}], "]"}], ",", " ", "1"}], "]"}], " ", 
             "*", "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"ih1", " ", ">", " ", "0"}], " ", "&&", " ", 
                RowBox[{"ih2", " ", ">", " ", "0"}]}], ",", " ", 
               RowBox[{"-", "1"}], ",", " ", "1"}], "]"}]}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"i1", ",", "i2"}], "}"}], "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"i12", ",", "s"}], "}"}]}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"ih1", ",", "0", ",", "1"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"ih2", ",", "0", ",", "1"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"il1", ",", "0", ",", 
            RowBox[{"n", "-", "1"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"il2", ",", "0", ",", 
            RowBox[{"n", "-", "1"}]}], "}"}]}], "]"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"newnp", ",", "newrls"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.620221113438829*^9, 3.620221136746541*^9}, {
  3.620221175619218*^9, 3.6202212241587877`*^9}, {3.6202212764781218`*^9, 
  3.620221354490531*^9}, {3.620221396216325*^9, 3.620221444890142*^9}, {
  3.6202215083234*^9, 3.620221679533091*^9}, {3.620221786424366*^9, 
  3.620221894921845*^9}, {3.620221980039419*^9, 3.620221984644515*^9}, {
  3.620222029550762*^9, 3.620222151767757*^9}, {3.620254150152104*^9, 
  3.6202541648752604`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDUnitMultList", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"Last", " ", "/@", " ", 
   RowBox[{"NestList", "[", 
    RowBox[{"CDUnitMultStep", ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], "}"}], ",", "np"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.620222176288931*^9, 3.6202222158705482`*^9}, {
  3.620251740722149*^9, 3.620251744860024*^9}, {3.6202534128043423`*^9, 
  3.6202534147998037`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDUnitMult", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"Last", " ", "@", " ", 
   RowBox[{"Nest", "[", 
    RowBox[{"CDUnitMultStep", ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}], "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], "}"}], ",", "np"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.620222176288931*^9, 3.6202222158705482`*^9}, {
  3.620253418414176*^9, 3.620253419779027*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDUnitMultTab", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"SparseArray", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", 
           RowBox[{"1", ",", "1"}], "]"}], "]"}], "+", "1"}], ",", 
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", 
           RowBox[{"1", ",", "2"}], "]"}], "]"}], "+", "1"}], ",", 
        RowBox[{
         RowBox[{"#", "[", 
          RowBox[{"[", 
           RowBox[{"2", ",", "1"}], "]"}], "]"}], "+", "1"}]}], "}"}], 
      "\[Rule]", 
      RowBox[{"#", "[", 
       RowBox[{"[", 
        RowBox[{"2", ",", "2"}], "]"}], "]"}]}], "&"}], " ", "/@", "  ", 
    RowBox[{"CDUnitMult", "[", "np", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.620224421606105*^9, 3.6202244738723717`*^9}, {
  3.6202534378830748`*^9, 3.6202534579877853`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TabMult", "[", 
   RowBox[{"tab_", ",", "x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{"y", ".", 
   RowBox[{"(", 
    RowBox[{"x", ".", "tab"}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.621235168619288*^9, 3.6212351802239733`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Jordan Algebras: Spin-Factor Algebras", "Section",
 CellChangeTimes->{{3.620975761357152*^9, 3.620975766094103*^9}, {
  3.620975806693899*^9, 3.620975809694519*^9}, {3.6209804999873466`*^9, 
  3.620980517017717*^9}, {3.62098057984317*^9, 3.6209805854566393`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SpnFac", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Join", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x", ".", "y"}], "}"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"First", "[", "x", "]"}], "*", 
      RowBox[{"Rest", "[", "y", "]"}]}], "+", 
     RowBox[{
      RowBox[{"First", "[", "y", "]"}], "*", 
      RowBox[{"Rest", "[", "x", "]"}]}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.620950333019642*^9, 3.620950338227653*^9}, {
  3.620950413369624*^9, 3.620950424836997*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SpnFacUnitTab", "[", "nv_", "]"}], " ", ":=", " ", 
  RowBox[{"SparseArray", "[", 
   RowBox[{"Flatten", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "1", ",", "1"}], "}"}], "\[Rule]", "1"}], ",", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{"i", ",", "i", ",", "1"}], "}"}], "\[Rule]", "1"}], ",", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", "i"}], "}"}], "\[Rule]", "1"}], ",", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"1", ",", "i", ",", "i"}], "}"}], "\[Rule]", "1"}]}], 
         "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "2", ",", 
          RowBox[{"nv", "+", "1"}]}], "}"}]}], "]"}]}], "}"}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.620975097232724*^9, 3.620975302851139*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SpnFacUnitTabVerify", "[", "nv_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"id", ",", "v1", ",", "v2", ",", "tab0", ",", "tab1"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"id", " ", "=", " ", 
      RowBox[{"IdentityMatrix", "[", 
       RowBox[{"nv", "+", "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"tab0", " ", "=", " ", 
      RowBox[{"SparseArray", "[", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"SparseArray", "[", 
          RowBox[{"SpnFac", "[", 
           RowBox[{"v1", ",", "v2"}], "]"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"v1", ",", "id"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"v2", ",", "id"}], "}"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"tab1", " ", "=", " ", 
      RowBox[{"SpnFacUnitTab", "[", "nv", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"tab1", "-", "tab0"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.620975418781208*^9, 3.6209754543882523`*^9}, {
  3.620975524990193*^9, 3.620975580549803*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Jordan Algebras: Hermitian Matrices of Division-Algebra Values\
\>", "Section",
 CellChangeTimes->{{3.620975910368774*^9, 3.620975930711954*^9}, {
  3.620980519136593*^9, 3.620980540952167*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Indices", " ", "in", " ", "upper", " ", "triangular", " ", "part", " ", 
   "of", " ", "matrix"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.621234640435874*^9, 3.621234653953971*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MatIdx", "[", 
   RowBox[{"lbl_", ",", "n_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"ix", ",", "i", ",", "j"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ix", " ", "=", " ", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"lbl", "[", 
         RowBox[{"i", ",", "j"}], "]"}], " ", "=", " ", 
        RowBox[{"ix", "++"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "n"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", 
         RowBox[{"i", "+", "1"}], ",", "n"}], "}"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.6210492115766077`*^9, 3.621049271330057*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"How", " ", "many", " ", 
   RowBox[{"values", "?"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.621100371572571*^9, 3.621100375431259*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"JMVals", "[", 
   RowBox[{"m_", ",", "np_"}], "]"}], " ", ":=", " ", 
  RowBox[{"m", " ", "+", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "/", "2"}], ")"}], "m", 
      RowBox[{"(", 
       RowBox[{"m", "-", "1"}], ")"}]}], ")"}], "*", 
    RowBox[{"2", "^", "np"}]}]}]}]], "Input",
 CellChangeTimes->{{3.621100376976967*^9, 3.621100396671723*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Diagonal", " ", "and", " ", "inverse", " ", "diagonal"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.621153235789956*^9, 3.62115324094557*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"JMDiag", "[", "x_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"xs", ",", "xv"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"xs", " ", "=", " ", 
      RowBox[{"First", "[", "x", "]"}]}], ";", " ", 
     RowBox[{"xv", " ", "=", " ", 
      RowBox[{"Rest", "[", "x", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"xs", " ", "+", " ", 
      RowBox[{"Append", "[", 
       RowBox[{"xv", ",", "0"}], "]"}], " ", "-", " ", 
      RowBox[{"Prepend", "[", 
       RowBox[{"xv", ",", "0"}], "]"}]}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.621153244884488*^9, 3.6211532770079737`*^9}, {
  3.6211566757821207`*^9, 3.6211566966760807`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"JMDInv", "[", "x_", "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"xs", ",", "xd", ",", "xv"}], "}"}], ",", "\[IndentingNewLine]", 
    
    RowBox[{
     RowBox[{"xs", " ", "=", " ", 
      RowBox[{"Mean", "[", "x", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"xd", " ", "=", " ", 
      RowBox[{"x", " ", "-", " ", "xs"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"xv", " ", "=", " ", 
      RowBox[{"Most", "[", 
       RowBox[{"Accumulate", "[", "xd", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Prepend", "[", 
      RowBox[{"xv", ",", "xs"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6211567002069263`*^9, 3.621156742334406*^9}, {
  3.621184210237438*^9, 3.621184217293385*^9}, {3.6211842954765167`*^9, 
  3.621184297564797*^9}, {3.621185373638026*^9, 3.621185386617075*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Off", "-", 
   RowBox[{
   "diagonal", " ", "with", " ", "conjugate", " ", "as", " ", 
    "appropriate"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.6212352082046633`*^9, 3.621235223607305*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"JMOfDg", "[", 
   RowBox[{"m_", ",", "n_", ",", "x_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"ixs", ",", "ix", ",", "i", ",", "j"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"MatIdx", "[", 
      RowBox[{"ixs", ",", "m"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Which", "[", 
        RowBox[{
         RowBox[{"j", ">", "i"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"ix", " ", "=", " ", 
           RowBox[{"ixs", "[", 
            RowBox[{"i", ",", "j"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"x", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"m", "+", 
              RowBox[{"n", "*", 
               RowBox[{"(", 
                RowBox[{"ix", "-", "1"}], ")"}]}], "+", "1"}], ";;", 
             RowBox[{"m", "+", 
              RowBox[{"n", "*", "ix"}]}]}], "]"}], "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"j", "<", "i"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"ix", " ", "=", " ", 
           RowBox[{"ixs", "[", 
            RowBox[{"j", ",", "i"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"CDConj", "[", 
           RowBox[{"x", "[", 
            RowBox[{"[", 
             RowBox[{
              RowBox[{"m", "+", 
               RowBox[{"n", "*", 
                RowBox[{"(", 
                 RowBox[{"ix", "-", "1"}], ")"}]}], "+", "1"}], ";;", 
              RowBox[{"m", "+", 
               RowBox[{"n", "*", "ix"}]}]}], "]"}], "]"}], "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"j", "\[Equal]", "i"}], ",", "\[IndentingNewLine]", "Null"}],
         "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"i", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "m"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.621234352689117*^9, 3.621234373021329*^9}, {
  3.621234405321403*^9, 3.621234423113681*^9}, {3.621234534584618*^9, 
  3.621234623002721*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"For", " ", "a", " ", "m", "*", "m", " ", "matrix"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.6210559676456747`*^9, 3.621055971428297*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"JdnMat", "[", 
   RowBox[{"m_", ",", "0", ",", "x_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "j", ",", "dg", ",", "od"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dg", " ", "=", " ", 
      RowBox[{"JMDiag", "[", 
       RowBox[{"Take", "[", 
        RowBox[{"x", ",", "m"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"od", " ", "=", " ", 
      RowBox[{"JMOfDg", "[", 
       RowBox[{"m", ",", "1", ",", "x"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"j", " ", "\[Equal]", " ", "i"}], ",", 
         RowBox[{"dg", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{"First", "[", 
          RowBox[{"od", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", "j"}], "]"}], "]"}], "]"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "m"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.621055855134983*^9, 3.621055876497136*^9}, {
  3.6210559257834787`*^9, 3.621055932826901*^9}, {3.621055988595805*^9, 
  3.621056014541218*^9}, {3.6210999954907293`*^9, 3.62110006687096*^9}, {
  3.621100144772398*^9, 3.621100201927038*^9}, {3.621229928165283*^9, 
  3.621229943302824*^9}, {3.621229994065289*^9, 3.621229995352415*^9}, {
  3.621234707443048*^9, 3.621234789507728*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"JdnMat", "[", 
   RowBox[{"m_", ",", "1", ",", "x_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "j", ",", "dg", ",", "od"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dg", " ", "=", " ", 
      RowBox[{"JMDiag", "[", 
       RowBox[{"Take", "[", 
        RowBox[{"x", ",", "m"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"od", " ", "=", " ", 
      RowBox[{"JMOfDg", "[", 
       RowBox[{"m", ",", "2", ",", "x"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"j", " ", "\[Equal]", " ", "i"}], ",", 
         RowBox[{"dg", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"1", ",", "I"}], "}"}], ".", 
          RowBox[{"od", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", "j"}], "]"}], "]"}]}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "m"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.621055855134983*^9, 3.621055876497136*^9}, {
  3.6210559257834787`*^9, 3.621055932826901*^9}, {3.621055988595805*^9, 
  3.621056014541218*^9}, {3.6210999954907293`*^9, 3.62110006687096*^9}, {
  3.621100144772398*^9, 3.621100201927038*^9}, {3.621229928165283*^9, 
  3.621229943302824*^9}, {3.621229994065289*^9, 3.621229995352415*^9}, {
  3.621234707443048*^9, 3.621234826299402*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"JdnMat", "[", 
   RowBox[{"m_", ",", "2", ",", "x_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "j", ",", "dg", ",", "od", ",", "id", ",", "qt"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dg", " ", "=", " ", 
      RowBox[{"JMDiag", "[", 
       RowBox[{"Take", "[", 
        RowBox[{"x", ",", "m"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"od", " ", "=", " ", 
      RowBox[{"JMOfDg", "[", 
       RowBox[{"m", ",", "4", ",", "x"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"id", " ", "=", " ", 
      RowBox[{"IdentityMatrix", "[", "2", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"qt", " ", "=", " ", 
      RowBox[{"Prepend", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "I"}], "*", 
         RowBox[{"Array", "[", 
          RowBox[{"PauliMatrix", ",", "3"}], "]"}]}], ",", "id"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"ArrayFlatten", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"j", " ", "\[Equal]", " ", "i"}], ",", 
          RowBox[{
           RowBox[{"dg", "[", 
            RowBox[{"[", "i", "]"}], "]"}], "*", "id"}], ",", 
          RowBox[{
           RowBox[{"od", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "j"}], "]"}], "]"}], ".", "qt"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "m"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"j", ",", "m"}], "}"}]}], "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.621055855134983*^9, 3.621055876497136*^9}, {
   3.6210559257834787`*^9, 3.621055932826901*^9}, {3.621055988595805*^9, 
   3.621056014541218*^9}, {3.6210999954907293`*^9, 3.62110006687096*^9}, {
   3.621100144772398*^9, 3.621100201927038*^9}, {3.621229928165283*^9, 
   3.621229943302824*^9}, {3.621229994065289*^9, 3.621229995352415*^9}, {
   3.621234707443048*^9, 3.621234826299402*^9}, {3.621234931860561*^9, 
   3.621235021926858*^9}, 3.621237796748398*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"JdnMatProd", "[", 
   RowBox[{"m_", ",", "np_", ",", "x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"mx", ",", "my"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"mx", " ", "=", " ", 
      RowBox[{"JdnMat", "[", 
       RowBox[{"m", ",", "np", ",", "x"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"my", " ", "=", " ", 
      RowBox[{"JdnMat", "[", 
       RowBox[{"m", ",", "np", ",", "y"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "/", "2"}], ")"}], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"mx", ".", "my"}], " ", "+", " ", 
         RowBox[{"my", ".", "mx"}]}], ")"}]}], " ", "//", " ", "Expand"}]}]}],
    "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.621232854007064*^9, 3.621232892330346*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"JdnProdMulTab", "[", 
   RowBox[{"m_", ",", "mtab_", ",", "x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "n", ",", "i", ",", "j", ",", "k", ",", "dgx", ",", "dgy", ",", "odx", 
      ",", "ody", ",", "dgxy", ",", "odxy"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", 
      RowBox[{"Length", "[", "mtab", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dgx", " ", "=", " ", 
      RowBox[{"JMDiag", "[", 
       RowBox[{"Take", "[", 
        RowBox[{"x", ",", "m"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dgy", " ", "=", " ", 
      RowBox[{"JMDiag", "[", 
       RowBox[{"Take", "[", 
        RowBox[{"y", ",", "m"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"odx", " ", "=", " ", 
      RowBox[{"JMOfDg", "[", 
       RowBox[{"m", ",", "n", ",", "x"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"ody", " ", "=", " ", 
      RowBox[{"JMOfDg", "[", 
       RowBox[{"m", ",", "n", ",", "y"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dgxy", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"dgx", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "*", 
          RowBox[{"dgy", "[", 
           RowBox[{"[", "i", "]"}], "]"}]}], "+", 
         RowBox[{"Sum", "[", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"k", "\[Equal]", "i"}], ",", "0", ",", 
             RowBox[{
              RowBox[{"odx", "[", 
               RowBox[{"[", 
                RowBox[{"i", ",", "k"}], "]"}], "]"}], ".", 
              RowBox[{"ody", "[", 
               RowBox[{"[", 
                RowBox[{"i", ",", "k"}], "]"}], "]"}]}]}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"k", ",", "m"}], "}"}]}], "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "m"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"odxy", " ", "=", " ", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"dgx", "[", 
              RowBox[{"[", "i", "]"}], "]"}], "+", 
             RowBox[{"dgx", "[", 
              RowBox[{"[", "j", "]"}], "]"}]}], ")"}], "*", 
           RowBox[{"ody", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "j"}], "]"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"dgy", "[", 
              RowBox[{"[", "i", "]"}], "]"}], "+", 
             RowBox[{"dgy", "[", 
              RowBox[{"[", "j", "]"}], "]"}]}], ")"}], "*", 
           RowBox[{"odx", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "j"}], "]"}], "]"}]}], " ", "+", " ", 
          RowBox[{"Sum", "[", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"k", "\[Equal]", "i"}], "||", 
               RowBox[{"k", "\[Equal]", "j"}]}], ",", "0", ",", 
              RowBox[{
               RowBox[{"TabMult", "[", 
                RowBox[{"mtab", ",", 
                 RowBox[{"odx", "[", 
                  RowBox[{"[", 
                   RowBox[{"i", ",", "k"}], "]"}], "]"}], ",", 
                 RowBox[{"ody", "[", 
                  RowBox[{"[", 
                   RowBox[{"k", ",", "j"}], "]"}], "]"}]}], "]"}], "+", 
               RowBox[{"TabMult", "[", 
                RowBox[{"mtab", ",", 
                 RowBox[{"ody", "[", 
                  RowBox[{"[", 
                   RowBox[{"i", ",", "k"}], "]"}], "]"}], ",", 
                 RowBox[{"odx", "[", 
                  RowBox[{"[", 
                   RowBox[{"k", ",", "j"}], "]"}], "]"}]}], "]"}]}]}], "]"}], 
            ",", 
            RowBox[{"{", 
             RowBox[{"k", ",", "m"}], "}"}]}], "]"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "m"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", 
           RowBox[{"i", "+", "1"}], ",", "m"}], "}"}]}], "]"}], "/", "2"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"JMDInv", "[", "dgxy", "]"}], ",", 
        RowBox[{"Flatten", "[", 
         RowBox[{"odxy", ",", "2"}], "]"}]}], "]"}], " ", "//", " ", 
      "Expand"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQB2IQHSChZ/Pl1yvHmHAxOxDNWXvYGUR7BKe5g+iux1x+
IFo1qzQARH8TPxcGokOcFkeC6HKvzGQQzaz2PR1EZ906VAiiRevVq0H0sj8n
joHoh4fOXALR0Wqi+l+B9EGrBiMQ7TfjlRmItvHXtQTRalcndIHoGRH75oBo
Jc69C0F00x+93SBa9EnJMRC9aNpSMH2gyuMuiBZQEr4HolvKYx+B6Pr2bDD9
z/3fZxC9o4H7C4hWtN3P9Q1I35jwnxtES939KguiT6jVK4JoHhZjNRA9y49Z
HUS/eGmtC6J3fdtiDqJX/vzkCqJNToi7geh3HVcTQHT4BYVEEO3Cu74ZRDOr
bgXTDMX7boHowxM8HoBoAMvQ5qU=
  "]],

Cell[BoxData[
 RowBox[{
  RowBox[{"JdnProd", "[", 
   RowBox[{"m_", ",", "np_", ",", "x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{"JdnProdMulTab", "[", 
   RowBox[{"m", ",", 
    RowBox[{"CDUnitMultTab", "[", "np", "]"}], ",", "x", ",", "y"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6212385075505447`*^9, 3.621238583713182*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"JdnProdVerify", "[", 
   RowBox[{"m_", ",", "np_", ",", "x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{"SparseArray", "[", 
   RowBox[{"Expand", "[", 
    RowBox[{
     RowBox[{"JdnMat", "[", 
      RowBox[{"m", ",", "np", ",", 
       RowBox[{"JdnProd", "[", 
        RowBox[{"m", ",", "np", ",", "x", ",", "y"}], "]"}]}], "]"}], "-", 
     RowBox[{"JdnMatProd", "[", 
      RowBox[{"m", ",", "np", ",", "x", ",", "y"}], "]"}]}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6212362616173286`*^9, 3.621236395427474*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"JdnProdVerifyLbls", "[", 
   RowBox[{"m_", ",", "np_", ",", "x_", ",", "y_"}], "]"}], " ", ":=", " ", 
  RowBox[{"JdnProdVerify", "[", 
   RowBox[{"m", ",", "np", ",", 
    RowBox[{"Array", "[", 
     RowBox[{"x", ",", 
      RowBox[{"JMVals", "[", 
       RowBox[{"m", ",", "np"}], "]"}]}], "]"}], ",", 
    RowBox[{"Array", "[", 
     RowBox[{"y", ",", 
      RowBox[{"JMVals", "[", 
       RowBox[{"m", ",", "np"}], "]"}]}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.621238600613666*^9, 3.621238645786046*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeUnitProdTab", "[", 
   RowBox[{"mult_", ",", "n_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"id", ",", "i", ",", "v1", ",", "v2"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"id", " ", "=", " ", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"SparseArray", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"{", "i", "}"}], "\[Rule]", "1"}], ",", 
          RowBox[{"{", "n", "}"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "n"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"SparseArray", "[", 
         RowBox[{"mult", "[", 
          RowBox[{"v1", ",", "v2"}], "]"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"v1", ",", "id"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"v2", ",", "id"}], "}"}]}], "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.621232954495241*^9, 3.62123302714599*^9}, {
  3.6212386892680187`*^9, 3.621238689843108*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"JdnUnitMultTab", "[", 
   RowBox[{"m_", ",", "np_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "mtab", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"mtab", " ", "=", " ", 
      RowBox[{"CDUnitMultTab", "[", "np", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"MakeUnitProdTab", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"JdnProdMulTab", "[", 
         RowBox[{"m", ",", "mtab", ",", "#1", ",", "#2"}], "]"}], "&"}], ",", 
       
       RowBox[{"JMVals", "[", 
        RowBox[{"m", ",", "np"}], "]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.621238708385988*^9, 3.6212387763232803`*^9}, {
  3.621238830459264*^9, 3.621238836948135*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SpnFacVerify", "[", "np_", "]"}], " ", ":=", " ", 
  RowBox[{"SparseArray", "[", 
   RowBox[{
    RowBox[{"JdnUnitMultTab", "[", 
     RowBox[{"2", ",", "np"}], "]"}], " ", "-", " ", 
    RowBox[{"SpnFacUnitTab", "[", 
     RowBox[{
      RowBox[{"JMVals", "[", 
       RowBox[{"2", ",", "np"}], "]"}], "-", "1"}], "]"}]}], "]"}]}]], "Input",\

 CellChangeTimes->{{3.6212388863470716`*^9, 3.62123892547539*^9}, {
  3.621238971484083*^9, 3.621239024892578*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Automorphisms", "Section",
 CellChangeTimes->{{3.621232949745143*^9, 3.621232951879931*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Given", " ", "unit", " ", "multiplication", " ", "tables"}], ",", 
   " ", 
   RowBox[{"find", " ", "automorphisms"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.6209504975053864`*^9, 3.620950502845476*^9}, {
  3.621239053011039*^9, 3.621239060317881*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"UnitMTabAuto", "[", 
   RowBox[{"mtab_", ",", "mat_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "mtr", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"mtr", " ", "=", " ", 
      RowBox[{"Transpose", "[", "mat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{
       RowBox[{"mtr", ".", 
        RowBox[{"Transpose", "[", 
         RowBox[{"mtr", ".", 
          RowBox[{"Transpose", "[", "mtab", "]"}]}], "]"}]}], " ", "-", " ", 
       RowBox[{"mtab", ".", "mtr"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.620950523319646*^9, 3.620950585976791*^9}, 
   3.620950634817398*^9, {3.621239128665971*^9, 3.621239133615058*^9}, {
   3.62123917052426*^9, 3.621239219612494*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"UnitMTabDiffAuto", "[", 
   RowBox[{"mtab_", ",", "mat_"}], "]"}], " ", ":=", "  ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "mtr", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"mtr", " ", "=", " ", 
      RowBox[{"Transpose", "[", "mat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{
       RowBox[{"mtr", ".", "mtab"}], " ", "+", " ", 
       RowBox[{"Transpose", "[", 
        RowBox[{"mtr", ".", 
         RowBox[{"Transpose", "[", "mtab", "]"}]}], "]"}], " ", "-", " ", 
       RowBox[{"mtab", ".", "mtr"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.620950523319646*^9, 3.62095063729877*^9}, {
  3.621239137379629*^9, 3.6212391438233433`*^9}, {3.6212392340241737`*^9, 
  3.621239261073415*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"FindMatsForConds", "[", 
   RowBox[{"mteqns_", ",", "n_"}], "]"}], " ", ":=", "  ", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"c", ",", "cmat", ",", "cmsol", ",", "vars", ",", "mats"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"cmat", " ", "=", " ", 
      RowBox[{"Array", "[", 
       RowBox[{"c", ",", 
        RowBox[{"{", 
         RowBox[{"n", ",", "n"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"cmsol", " ", "=", " ", 
      RowBox[{"cmat", " ", "/.", " ", 
       RowBox[{"ToRules", "[", 
        RowBox[{"Reduce", "[", 
         RowBox[{"Thread", "[", 
          RowBox[{
           RowBox[{"Flatten", "[", 
            RowBox[{"mteqns", "[", "cmat", "]"}], "]"}], "\[Equal]", "0"}], 
          "]"}], "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"vars", " ", "=", " ", 
      RowBox[{"Flatten", "[", "cmat", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"mats", " ", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"SparseArray", "[", 
         RowBox[{"D", "[", 
          RowBox[{"cmsol", ",", "#"}], "]"}], "]"}], "&"}], " ", "/@", " ", 
       "vars"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"Select", "[", 
       RowBox[{"mats", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Union", "[", 
           RowBox[{"Flatten", "[", 
            RowBox[{"Normal", "[", "#", "]"}], "]"}], "]"}], "=!=", 
          RowBox[{"{", "0", "}"}]}], "&"}]}], "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.621240314983305*^9, 3.621240437937214*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"UnitMTabAutoGen", "[", "mtab_", "]"}], " ", ":=", " ", 
  RowBox[{"FindMatsForConds", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"UnitMTabDiffAuto", "[", 
      RowBox[{"mtab", ",", "#"}], "]"}], "&"}], ",", 
    RowBox[{"Length", "[", "mtab", "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.621239762847768*^9, 3.6212399166152678`*^9}, {
  3.621239950311708*^9, 3.621240000768309*^9}, {3.621240040074666*^9, 
  3.621240041265751*^9}, {3.621240077083707*^9, 3.6212401050822697`*^9}, {
  3.6212404506811447`*^9, 3.621240463134248*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Expected", " ", "for", " ", "both", " ", "the", " ", "division"}],
    "-", 
   RowBox[{"algebra", " ", "and", " ", "the", " ", "Jordan"}], "-", 
   RowBox[{"matrix", " ", "automorphisms"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.621240108775279*^9, 3.621240125704444*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TestFirstZeros", "[", "gens_", "]"}], " ", ":=", " ", 
  RowBox[{"SparseArray", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"SparseArray", "[", 
      RowBox[{"gens", "[", 
       RowBox[{"[", 
        RowBox[{";;", ",", ";;", ",", "1"}], "]"}], "]"}], "]"}], ",", 
     RowBox[{"SparseArray", "[", 
      RowBox[{"gens", "[", 
       RowBox[{"[", 
        RowBox[{";;", ",", "1", ",", ";;"}], "]"}], "]"}], "]"}]}], "}"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6212401571583023`*^9, 3.6212402061454983`*^9}, {
  3.621240646823831*^9, 3.621240650214902*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SelectRestOfMats", "[", "gens_", "]"}], " ", ":=", " ", 
  RowBox[{"SparseArray", "[", 
   RowBox[{"gens", "[", 
    RowBox[{"[", 
     RowBox[{";;", ",", 
      RowBox[{"2", ";;"}], ",", 
      RowBox[{"2", ";;"}]}], "]"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.621240229711506*^9, 3.621240250881938*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Invariant", " ", "matrices", " ", "for", " ", "the", " ", "generators"}], 
  " ", "*)"}]], "Input",
 CellChangeTimes->{{3.621240546914838*^9, 3.621240554909225*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"GenInvarMats", "[", "gens_", "]"}], " ", ":=", " ", 
  RowBox[{"FindMatsForConds", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Transpose", "[", 
       RowBox[{"#", ".", 
        RowBox[{"Transpose", "[", "gens", "]"}]}], "]"}], " ", "-", " ", 
      RowBox[{"gens", ".", "#"}]}], "&"}], ",", 
    RowBox[{
     RowBox[{"Dimensions", "[", "gens", "]"}], "[", 
     RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.621240557947083*^9, 3.621240613981846*^9}}]
}, Closed]]
},
WindowSize->{808, 696},
WindowMargins->{{Automatic, 204}, {46, Automatic}},
FrontEndVersion->"10.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (September 9, \
2014)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[558, 20, 2250, 75, 1132, "Text"],
Cell[CellGroupData[{
Cell[2833, 99, 158, 2, 64, "Section"],
Cell[2994, 103, 195, 4, 28, "Input"],
Cell[3192, 109, 306, 8, 28, "Input"],
Cell[3501, 119, 229, 5, 28, "Input"],
Cell[3733, 126, 3907, 92, 284, "Input"],
Cell[7643, 220, 638, 17, 28, "Input"],
Cell[8284, 239, 576, 16, 28, "Input"],
Cell[8863, 257, 931, 27, 46, "Input"],
Cell[9797, 286, 275, 7, 28, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[10109, 298, 269, 3, 50, "Section"],
Cell[10381, 303, 559, 16, 28, "Input"],
Cell[10943, 321, 976, 28, 63, "Input"],
Cell[11922, 351, 1219, 30, 114, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[13178, 386, 204, 4, 50, "Section"],
Cell[13385, 392, 236, 5, 28, "Input"],
Cell[13624, 399, 796, 22, 80, "Input"],
Cell[14423, 423, 189, 4, 28, "Input"],
Cell[14615, 429, 430, 13, 28, "Input"],
Cell[15048, 444, 194, 4, 28, "Input"],
Cell[15245, 450, 773, 19, 80, "Input"],
Cell[16021, 471, 932, 22, 114, "Input"],
Cell[16956, 495, 246, 6, 28, "Input"],
Cell[17205, 503, 2209, 56, 233, "Input"],
Cell[19417, 561, 192, 4, 28, "Input"],
Cell[19612, 567, 1566, 38, 97, "Input"],
Cell[21181, 607, 1620, 40, 97, "Input"],
Cell[22804, 649, 2141, 51, 131, "Input"],
Cell[24948, 702, 943, 26, 97, "Input"],
Cell[25894, 730, 4949, 128, 250, "Input"],
Cell[30846, 860, 342, 8, 28, "Input"],
Cell[31191, 870, 569, 14, 46, "Input"],
Cell[31763, 886, 556, 14, 46, "Input"],
Cell[32322, 902, 1173, 33, 80, "Input"],
Cell[33498, 937, 784, 20, 80, "Input"],
Cell[34285, 959, 495, 13, 28, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[34817, 977, 98, 1, 50, "Section"],
Cell[34918, 980, 318, 7, 28, "Input"],
Cell[35239, 989, 841, 20, 80, "Input"],
Cell[36083, 1011, 846, 20, 80, "Input"],
Cell[36932, 1033, 1687, 44, 131, "Input"],
Cell[38622, 1079, 572, 12, 28, "Input"],
Cell[39197, 1093, 339, 7, 28, "Input"],
Cell[39539, 1102, 605, 16, 46, "Input"],
Cell[40147, 1120, 351, 9, 28, "Input"],
Cell[40501, 1131, 214, 5, 28, "Input"],
Cell[40718, 1138, 534, 14, 46, "Input"]
}, Closed]]
}
]
*)

(* End of internal cache information *)
